/**
 Copyright (c) 2004-2006 Honda Research Institute Japan, Co., Ltd.

\file PF_Init.h
\brief \~Japanese Post filter ϐAf[^̏
\author Sachiko Suzuki
\date 24 Jan 2007
\Version $Id: PF_Init.h 453 2009-03-17 04:44:42Z tanaka $
*/
// ---------------------------------------------------------------------
#ifndef	PFInit_h_defined
#define	PFInit_h_defined
// ---------------------------------------------------------------------
#include <stdio.h>
#include <string.h>
#include <ctype.h>
//#include "MICARYHA_struct.h"	// 2008/11/11 deleted by N.Tanaka MICARY_struct.hoRŌĂ
#include "MICARY_struct.h"
#include "MICARY_PF_Struct.h"
#include "PF_values.h"
#include "ArrayInit.h"

// 2007/07/13 added by S.Suzuki C++CĂяo
#ifdef __cplusplus
extern "C" {
#endif

// ---------------------------------------------------------------------

/**
 \brief \~English It initialize the parameter for Post filter.
 \brief \~Japanese ϐ̏(1:I, 0:G[)
 \param psPrmMCRA \English Setting parameter for MCRA
 \~Japanese MCRAݒp[^
 \param psPrmLeak \English Setting parameter for Leak noise
 \~Japanese Rݒp[^
 \param psPrmRev \English Setting parameter for Reverb factor
 \~Japanese cݒp[^
 \param psPrmSN \English Setting parameter for estimate SN ratio
 \~Japanese SN䐄ݒp[^
 \param psPrmVoiceP \English Setting parameter for Voice Probability
 \~Japanese ݊mݒp[^
 \param psPrmGain \English Setting parameter for estimate gain
 \~Japanese œKQCݒp[^ 
*/
int InitPFsPrm(MICARY_PrmMCRA *psPrmMCRA , MICARY_PrmLeakNoise *psPrmLeak ,
			   MICARY_PrmRevNoise *psPrmRev , MICARY_PrmEstSN *psPrmSN ,
			   MICARY_PrmVoiceP *psPrmVoiceP , MICARY_PrmGain *psPrmGain) ;

// ---------------------------------------------------------------------
/**
 \brief \~English It allocates the saved data(MCRA) for Post filter.
 \brief \~Japanese f[^ۑl(MCRA)̊m(1:I, 0:G[)
 \param psSavedMCRA \English Saved data for MCRA
 \~Japanese MCRAۑl
 \param iUpperFreq \English Upper frequency [Line]
 \~Japanese gC
 \param iLowerFreq \English Lower frequency [Line]
 \~Japanese gC
 \param Nch \English Number of source position
 \~Japanese 
*/
int AllocSavedMCRA(MICARY_SavedMCRA *psSavedMCRA ,HA_Integer iUpperFreq ,
				  HA_Integer iLowerFreq ,HA_Integer Nch) ;

// ---------------------------------------------------------------------
/**
 \brief \~English It frees the saved data(MCRA) for Post filter.
 \brief \~Japanese f[^ۑl(MCRA)̊J(1:I, 0:G[)
 \param psSavedMCRA \English Saved data for MCRA
 \~Japanese MCRAۑl
*/
int FreeSavedMCRA(MICARY_SavedMCRA *psSavedMCRA) ;
//
//// ---------------------------------------------------------------------
///**
// \brief \~English It frees the saved data(EstSN) for Post filter.
// \brief \~Japanese f[^ۑl(EstSN)̊J(1:I, 0:G[)
// \param psSavedEstSN \English Saved data for EstSN
// \~Japanese EstSNۑl
//*/
//int FreeSavedEstSN(MICARY_SavedEstSN *psSavedEstSN) ;


// ---------------------------------------------------------------------
/**
 \brief \~English It allocates the saved data(estimate Rev) for Post filter.
 \brief \~Japanese f[^ۑl(Rev)̊m(1:I, 0:G[)
 \param psSavedRev \English saved data for Reverb factor
 \~Japanese cۑl
 \param iUpperFreq \English Upper frequency [Line]
 \~Japanese gC
 \param iLowerFreq \English Lower frequency [Line]
 \~Japanese gC
 \param Nch \English Number of source position
 \~Japanese 
*/
int AllocSavedRev(MICARY_SavedEstRev *psSavedRev ,HA_Integer iUpperFreq ,
				  HA_Integer iLowerFreq ,HA_Integer Nch) ;

// ---------------------------------------------------------------------
/**
 \brief \~English It frees the saved data(estimate rev) for Post filter.
 \brief \~Japanese f[^ۑl(Rev)̊J(1:I, 0:G[)
 \param psSavedRev \English Saved data for Reverb factor
 \~Japanese cۑl
*/
int FreeSavedRev(MICARY_SavedEstRev *psSavedRev) ;

// ----------------------------------------------------------------------
/**
 \brief \~English It allocates the saved data(estimate SN ratio) for Post filter.
 \brief \~Japanese f[^ۑl(SN ratio)̊m(1:I, 0:G[)
 \param psSavedSN \English saved data for SN ratio
 \~Japanese SNۑl
 \param iUpperFreq \English Upper frequency [Line]
 \~Japanese gC
 \param iLowerFreq \English Lower frequency [Line]
 \~Japanese gC
 \param Nch \English Number of source position
 \~Japanese 
*/
int AllocSavedSN(MICARY_SavedEstSN *psSavedSN ,HA_Integer iUpperFreq ,
				  HA_Integer iLowerFreq ,HA_Integer Nch) ;
// ----------------------------------------------------------------------
/**
 \brief \~English It frees the saved data(estimate SN ratio) for Post filter.
 \brief \~Japanese f[^ۑl(SN ratio)̊J(1:I, 0:G[)
 \param psSavedSN \English saved data for SN ratio
 \~Japanese SNۑl
*/
int FreeSavedSN(MICARY_SavedEstSN *psSavedSN ) ;
// ----------------------------------------------------------------------
/**
 \brief \~English It allocates the saved data(Voice Probability) for Post filter.
 \brief \~Japanese f[^ۑl(Voice Probability)̊m(1:I, 0:G[)
 \param psSavedVoiceP \English saved data for Voice Probability
 \~Japanese ݊mۑl
 \param iUpperFreq \English Upper frequency [Line]
 \~Japanese gC
 \param iLowerFreq \English Lower frequency [Line]
 \~Japanese gC
 \param Nch \English Number of source position
 \~Japanese 
*/
int AllocSavedVoiceP(MICARY_SavedVoiceP *psSavedVoiceP ,HA_Integer iUpperFreq ,
				  HA_Integer iLowerFreq ,HA_Integer Nch) ;
// ----------------------------------------------------------------------
/**
 \brief \~English It frees the saved data(Voice Probability) for Post filter.
 \brief \~Japanese f[^ۑl(Voice Probability)̊J(1:I, 0:G[)
 \param psSavedVoiceP \English saved data for Voice Probability
 \~Japanese ݊mۑl
*/
int FreeSavedVoiceP(MICARY_SavedVoiceP *psSavedVoiceP ) ;
// ----------------------------------------------------------------------
// ---------------------------------------------------------------------
/**
 \brief \~English It allocates the saved data(Initialized MCRA) for Post filter.
 \brief \~Japanese f[^ۑl(MCRȀl)̊m(1:I, 0:G[)
 \param psSavedInitMCRA \English Saved data for MCRA
 \~Japanese MCRAۑl
 \param iUpperFreq \English Upper frequency [Line]
 \~Japanese gC
 \param iLowerFreq \English Lower frequency [Line]
 \~Japanese gC
 \param Nch \English Number of source position
 \~Japanese 
*/
int AllocSavedInitMCRA(MICARY_SavedInitMCRA *psSavedInitMCRA ,HA_Integer iUpperFreq ,
				  HA_Integer iLowerFreq ,HA_Integer Nch)  ;

// ---------------------------------------------------------------------
/**
 \brief \~English It frees the saved data(Initialized MCRA) for Post filter.
 \brief \~Japanese f[^ۑl(MCRȀl)̊J(1:I, 0:G[)
 \param psSavedInitMCRA \English Saved data for MCRA
 \~Japanese MCRAۑl
*/
int FreeSavedInitMCRA(MICARY_SavedInitMCRA *psSavedInitMCRA)  ;

// ---------------------------------------------------------------------
/**
 \brief \~English It assigns the initialized MCRA parameter  for Post filter.
 \brief \~Japanese MCRȀlpp[^̐ݒ(1:I, 0:G[)
 \param psPrmInitMCRA \English Parameter structure for Initialized MCRA
 \~Japanese MCRAp[^
*/
int InitPrmInMCRA(MICARY_PrmInitMCRA *psPrmInitMCRA) ;

// ---------------------------------------------------------------------
// ---------------------------------------------------------------------
/**
 \brief \~English It initializes temporary data for Post filter.
 \brief \~Japanese ۑlf[^̏(1:I, 0:G[)
 \param psSavedMCRA \English Saved data structure for Initialized MCRA
 \~Japanese MCRAf[^\
 \param psSavedRev \English Saved data structure for Initialized REV
 \~Japanese REVpf[^\
 \param psSavedSN \English Saved data structure for Initialized SN
 \~Japanese SNpf[^\
 \param psSavedVoiceP \English Saved data structure for Initialized VoiceP
 \~Japanese mf[^\
 \param faNoise \English Estimated noise power data array
 \~Japanese mCYp[
 \param noise_compense \English parameter of noise_compense
 \~Japanese Gp[p[^
 \param iLowerFreq \English Lower Frequency line index
 \~Japanese gCԍ
 \param iUpperFreq \English Upper frequency line index
 \~Japanese gCԍ
 \param Nsrc \English Number of Source
 \~Japanese 
 \param Nmic \English Number of microphone
 \~Japanese }CN`l
*/
int PFInitTmp(MICARY_SavedMCRA *psSavedMCRA ,MICARY_SavedEstRev *psSavedRev,MICARY_SavedEstSN *psSavedSN ,
			  MICARY_SavedVoiceP *psSavedVoiceP,MICARY_FloatArray *faNoise ,HA_Float noise_compense,
			  HA_Integer iLowerFreq ,HA_Integer iUpperFreq,HA_Integer Nsrc,HA_Integer Nmic) ;



// 2008/12/5 added by K.Morio
/**
 \brief \~English It initializes temporary data for Post filter.
 \brief \~Japanese ۑlf[^̏(1:I, 0:G[)
 \param psSavedMCRA \English Saved data structure for Initialized MCRA
 \~Japanese MCRAf[^\
 \param psSavedRev \English Saved data structure for Initialized REV
 \~Japanese REVpf[^\
 \param psSavedSN \English Saved data structure for Initialized SN
 \~Japanese SNpf[^\
 \param psSavedVoiceP \English Saved data structure for Initialized VoiceP
 \~Japanese mf[^\
 \param faNoise \English Initial State Noise data array
 \~Japanese Gl
 \param iLowerFreq \English Lower Frequency line index
 \~Japanese gCԍ
 \param iUpperFreq \English Upper frequency line index
 \~Japanese gCԍ


*/
int PFInitTmp2(MICARY_SavedMCRA *psSavedMCRA ,MICARY_SavedEstRev *psSavedRev,MICARY_SavedEstSN *psSavedSN ,
			  MICARY_SavedVoiceP *psSavedVoiceP,MICARY_FloatArray *pfCalcStateNoise,
			  HA_Integer iLowerFreq ,HA_Integer iUpperFreq);

// 2008/12/5 added by K.Morio
/**
 \brief \~English It calcurates Initial state noise power.
 \brief \~Japanese Gl̎Zo(1:I, 0:G[)
 \param faNoise \English Estimated noise power data array
 \~Japanese mCYp[
 \param noise_compense \English parameter of noise_compense
 \~Japanese Gp[p[^
  \param faNoise \English Initial State Noise data array
 \~Japanese Gl
 \param iLowerFreq \English Lower Frequency line index
 \~Japanese gCԍ
 \param iUpperFreq \English Upper frequency line index
 \~Japanese gCԍ
 \param Nsrc \English Number of Source
 \~Japanese 
 \param Nmic \English Number of microphone
 \~Japanese }CN`l


*/
int CalcInitStateNoise(MICARY_FloatArray faNoise ,HA_Float noise_compense,MICARY_FloatArray *pfCalcStateNoise,
			  HA_Integer iLowerFreq ,HA_Integer iUpperFreq,HA_Integer Nsrc,HA_Integer Nmic);



/**
 \brief \~English It calculates minimum sum (PF_init_state_noise.c/PF_est_state_noise.c).
 \brief \~Japanese ϐAAB̍ŏl̍Zo(PF_init_state_coise.c/PF_est_state_noise.c).
 \param A \English Variable A
 \~Japanese ϐA
 \param B \English Variable B
 \~Japanese ϐB
*/
HA_Float minmat(HA_Float A ,HA_Float B) ;






// 2008/08/06 added by N.Tanaka
/**
 \brief \~Japanese Rꐄpp[^̐ݒ
 \param psPrmLeakNoise \~English Parameter struct for estimating leak noise
 \~Japanese Rꐄpp[^̐ݒ
 \param fLeak_factor \~English Leak factor
 \~Japanese RW
 \param fOver_cancel \~English Over cancel value
 \~Japanese I[o[LZl
*/
int Input_PF_PrmLeak(MICARY_PrmLeakNoise *psPrmLeakNoise, float fLeak_factor,float fOver_cancel) ;


#ifdef __cplusplus
}
#endif
// ----------------------------------------------------------------------------------------

#endif // PF_CmdLine_h_defined

